#ifndef RELATION_HH
#define RELATION_HH

#include <variant> // C++17, compile with -std=c++17 // does not yet work with clang++
#include <string>
#include <vector>
#include <fstream>
#include <sstream>
#include <stdexcept>

/*
 * This file contains the implementation of a simple storage layer.
 * - To insert a new relation R, create a R.schema and R.data file in the
 *   database directory. See database/test.data and database/test.schema for an example.
 * - To load a relation from disk to main memory, create a Relation object.
 */

// type definitions for attribute values, tuples, relations and schemata of relations
using attrVal_t  = std::variant<int, double, std::string>; // cf.: http://en.cppreference.com/w/cpp/utility/variant
using tuple_t    = std::vector<attrVal_t>;
using relation_t = std::vector<tuple_t>;
using schema_t   = std::vector<std::string>;


// function declerations for functions to print tuples and attribute values
void printTuple    (const tuple_t& tuple, std::ostream& os);
void printAttrVal  (const attrVal_t & attrVal, std::ostream& os);


// Relation class reads relation from file to main memory
// and stores the data
class Relation {
  public:
    Relation (const std::string& aRelationDescriptor, char delimiter);
  public:
    const schema_t&   getSchema ();
    const relation_t& getData   ();
    void              print     (std::ostream& os);
  private:
    void      readSchema  (const std::string& schemaDescriptor,
                           char               delimiter);
    void      readData    (const std::string& dataDescriptor,
                           const char         delimiter);
    tuple_t   readTuple   (const std::string& tupleString,
                           const char         delimiter);
    attrVal_t readAttrVal (std::istringstream& attrValStream,
                           const unsigned      attrIndex,
                           const char          delimiter);
  private:
    relation_t  _data;
    schema_t    _schema;
    std::string _name;
};

#endif
